package xyz.imaf6971.volgait.controllers;

import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import xyz.imaf6971.volgait.dto.AdvertisementDto;
import xyz.imaf6971.volgait.dto.PublishAdvertisementDto;
import xyz.imaf6971.volgait.service.AdvertisementService;

import javax.validation.Valid;
import java.net.URI;
import java.util.List;

@RestController
@RequestMapping("/api/advertisement")
public class AdvertisementController {

    private final AdvertisementService advertisementService;

    public AdvertisementController(AdvertisementService advertisementService) {
        this.advertisementService = advertisementService;
    }

    @PostMapping
    public ResponseEntity<?> publishAdvertisement(@RequestBody @Valid PublishAdvertisementDto publishDto) {
        var publishId = advertisementService.publish(publishDto);
        return ResponseEntity.created(URI.create("/api/advertisement/" + publishId)).build();
    }

    @GetMapping
    public List<AdvertisementDto> listAdvertisements() {
        return advertisementService.listAll();
    }


    @GetMapping("/{advertisementId}")
    public AdvertisementDto getAdvertisementById(@PathVariable Long advertisementId) {
        return advertisementService.getAdvertisementById(advertisementId);
    }

    @PutMapping("/{advertisementId}")
    public void changeAdvertisement(@RequestBody PublishAdvertisementDto changes, @PathVariable Long advertisementId) {
        advertisementService.changeById(changes, advertisementId);
    }

    @DeleteMapping("/{advertisementId}")
    public void deleteAdvertisement(@PathVariable Long advertisementId) {
        advertisementService.deleteAdvertisementById(advertisementId);
    }
}
