package xyz.imaf6971.volgait.service.impl;

import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import xyz.imaf6971.volgait.dto.UserDto;
import xyz.imaf6971.volgait.exceptions.RoleNotFoundException;
import xyz.imaf6971.volgait.exceptions.UsernameAlreadyTakenException;
import xyz.imaf6971.volgait.models.Role;
import xyz.imaf6971.volgait.models.User;
import xyz.imaf6971.volgait.repository.RoleRepository;
import xyz.imaf6971.volgait.repository.UserRepository;
import xyz.imaf6971.volgait.service.RegistrationService;

import javax.transaction.Transactional;

@Service
public class RegistrationServiceImpl implements RegistrationService {

    private final UserRepository userRepository;

    private final PasswordEncoder passwordEncoder;

    private final RoleRepository roleRepository;

    public RegistrationServiceImpl(UserRepository userRepository, PasswordEncoder passwordEncoder, RoleRepository roleRepository) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
        this.roleRepository = roleRepository;
    }

    @Override
    @Transactional
    public void register(UserDto userDto) {
        checkUserExists(userDto);
        registerNewUser(userDto);
    }

    private void checkUserExists(UserDto userDto) {
        if (userExists(userDto.username())) {
            throw new UsernameAlreadyTakenException("User with username " + userDto.username() + " already exists!");
        }
    }

    private void registerNewUser(UserDto userDto) {
        var user = new User();
        user.setUsername(userDto.username());
        user.setPassword(passwordEncoder.encode(userDto.password()));
        user.setActive(true);
        user.setRole(getDefaultRole());
        userRepository.saveAndFlush(user);
    }

    private Role getDefaultRole() {
        return roleRepository.findByTitle("ROLE_USER")
                .orElseThrow(() -> new RoleNotFoundException("USER role not found"));
    }

    private boolean userExists(String email) {
        return userRepository.existsByUsername(email);
    }

}
