package xyz.imaf6971.volgait.service.impl;

import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import xyz.imaf6971.volgait.exceptions.UsernameAlreadyTakenException;
import xyz.imaf6971.volgait.models.User;
import xyz.imaf6971.volgait.repository.UserRepository;
import xyz.imaf6971.volgait.service.UserService;

@Service
public class UserServiceImpl implements UserService {

    private final UserRepository userRepository;

    public UserServiceImpl(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    @Override
    public void changeUsername(String oldUsername, String newUsername) {
        var user = getUserByUsername(oldUsername);
        if (!userRepository.existsByUsername(newUsername)) {
            user.setUsername(newUsername);
            userRepository.save(user);
            return;
        }
        throw new UsernameAlreadyTakenException("User with username " + newUsername + " already exists");
    }

    @Override
    public void deleteUser(String username) {
        var user = getUserByUsername(username);
        user.setActive(false);
    }

    @Override
    public User getUserByUsername(String username) {
        return userRepository.findByUsername(username)
                .orElseThrow(() -> new UsernameNotFoundException("User with username " + username + " not found"));
    }

    @Override
    public User getCurrentUser() {
        return getUserByUsername(getCurrentUsername());
    }

    private String getCurrentUsername() {
        var auth = SecurityContextHolder.getContext().getAuthentication();
        return auth.getName();
    }
}
